<?php
/**
 * The Horde_Cache_memcache:: class provides a memcached implementation of the
 * Horde caching system.
 * memcached website: http://www.danga.com/memcached/
 *
 * $Horde: framework/Cache/Cache/memcache.php,v 1.7 2006/03/11 06:24:48 chuck Exp $
 *
 * Optional parameters:<pre>
 *   'hostspec'     - The memcached host to connect to
 *   'port'         - The port where memcache is listening to.
 *   'persistent'   - Use persistent DB connections?
 *   'compression'  - Compress data inside memcache? Needs gzip support inside
 *                    memcache.
 * </pre>
 *
 * Copyright 2006 Duck <duck@obala.net>
 *
 * See the enclosed file COPYING for license information (LGPL). If you
 * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
 *
 * @author  Duck <duck@obala.net>
 * @since   Horde 3.2
 * @package Horde_Cache
 */
class Horde_Cache_memcache extends Horde_Cache {

    /**
     * Handler for the current memcached connection.
     *
     * @var resource
     */
    var $_db;

    /**
     * Whether to use compression.
     *
     * @var integer
     */
    var $_compress = 0;

    /**
     * Construct a new Horde_Cache_memcache object.
     *
     * @param array $params  Parameter array.
     */
    function Horde_Cache_memcache($params = array())
    {
        $this->_compress = $params['compression'] ? MEMCACHE_COMPRESSED : 0;
        if (empty($params['persistent'])) {
            $connect_persistent = false;
        } else {
            $connect_persistent = true;
        }

        $this->_db = new Memcache;
        for ($i = 0, $n = count($params['hostspec']); $i < $n; ++$i) {
            $this->_db->addServer($params['hostspec'][$i], $params['port'][$i], $connect_persistent);
        }
    }

    /**
     * Attempts to retrieve cached data from the memcache and return it to
     * the caller.
     *
     * @param string  $key       Cache key to fetch.
     * @param integer $lifetime  Lifetime of the data in seconds.
     *
     * @return mixed  Cached data, or false if none was found.
     */
    function get($key, $lifetime = 1)
    {
        $this->_setExpire($key, $lifetime);
        return $this->_db->get($key);
    }

    /**
     * Attempts to store data to the memcache.
     *
     * @param string $key   Cache key.
     * @param mixed  $data  Data to store in the cache.
     *
     * @return boolean  True on success, false on failure.
     */
    function set($key, $data)
    {
        $this->_db->set($key . '_expire', time(), 0, 0);
        return $this->_db->set($key, $data, $this->_compress, 0);
    }

    /**
     * Attempts to directly output cached data from the memcache.
     *
     * @param string  $key       Cache key to output.
     * @param integer $lifetime  Lifetime of the data in seconds.
     *
     * @return mixed  Cached data, or false if none was found.
     */
    function output($key, $lifetime = 1)
    {
        $this->_setExpire($key, $lifetime);
        if (($data = $this->_db->get($key)) !== false) {
            echo $data;
            return true;
        } else {
            return false;
        }
    }

    /**
     * Checks if a given key exists in the cache, valid for the given
     * lifetime. If it exists but is expired, delete the cache entry.
     *
     * @param string  $key       Cache key to check.
     * @param integer $lifetime  Lifetime of the key in seconds.
     *
     * @return boolean  Existance.
     */
    function exists($key, $lifetime = 1)
    {
        $this->_setExpire($key, $lifetime);
        return $this->_db->get($key) === false ? false : true;
    }

    /**
     * Expire any existing data for the given key.
     *
     * @param string $key  Cache key to expire.
     *
     * @return boolean  Success or failure.
     */
    function expire($key)
    {
        $this->_db->delete($key . '_expire');
        return $this->_db->delete($key);
    }

    /**
     * Set expire time on each call since memcache sets it on cache creation.
     *
     * @access private
     *
     * @param string  $key   Cache key to expire.
     * @param integer $lifetime  Lifetime of the data in seconds.
     */
    function _setExpire($key, $lifetime)
    {
        if ($lifetime == 0) {
            // don't expire
            return true;
        }

        $expire = $this->_db->get($key . '_expire');
        if ($expire == 0) {
            // alreay checked
            return true;
        }

        // set prune period
        if ($expire+$lifetime<time()) {
            $lifetime = 0;
        } else {
            $this->_db->replace($key . '_expire', 0);
            $lifetime = ($expire+$lifetime) - time();
        }

        $this->_db->delete($key, $lifetime);
        $this->_db->delete($key . '_expire', $lifetime);

        return $lifetime;
    }

}
